# This script reads in a phylogeny, ancestral trait values with their ses
# for two traits, and node ages. Then it bootstraps the distribution of
# ancestral values and calculates the weighted divergence age for each 
# trait, and the difference between these values. This provides a 
# p-value on the DOT statistic.
#
# David Ackerly 2005 dackerly@berkeley.edu

# Set BATCH = 1 if this script is embedded in a larger loop; it eliminates 
# calculation of some extra variables and provides a one line output format
BATCH <- 0

#Number of reps for bootstrapping
Nreps <- 1

#load libraries
library(ape)
library(msm)
source('picfixed.R')

# Call trancml - awk script that reads phylo and traits, runs ancml, and
# converts ancml output files to 'dot.in', which is used below by this
# program.
#
# print('call trancml')
system('./trancml > dot.in')
#print('exit trancml')

# Set REPLACE = TRUE to sample with replacement in null models
REPLACE <- FALSE

# CANCUT?
taxon.names <- 1 # 1 if first column of dot.in is taxon names matching treefile
		 # 0 if first column is other names

#read tree in simple newick format; read.tree function is from ape package
tree <- read.tree(file='phylo')

# Set flag BLzero = 1 if tree has zero length branches; = 0 if not
ifelse(min(tree$edge.length) == 0,
	{
	BLzero <- 1
	print('WARNING: tree has zero length branches')
	},
	BLzero <- 0)

#tree dimensions
NTips <- max(as.numeric(tree$edge))
NTot <- 2*NTips-1
Ncont <- NTips-1
qq<-((1:Ncont)-0.5)/(Ncont) #quantiles for truncated normal prob plots

#Read dot.in file, including ancestral values with se's for internal
#nodes. The order of nodes in dot.in must not be changed after it is'
#created by trancml - it exactly matches node order in variable tree,
#created by read.tree command above. Order of tips can be determined
#running read.tree and then examining tree$tip.label. Order of internals
#matches order in phylocom: starts at root, and then reads left
#and up at each node, then down and to the right, until tree is
#traversed

T <- read.table('dot.in',header=TRUE,sep='\t')
#print(T)

# Partition traits data for tips
Ttips <- T[T['down']=='.']
dim(Ttips) <- c(length(Ttips)/ncol(T),ncol(T))
colnames(Ttips) <- colnames(T)
rownames(Ttips) <- Ttips[,'taxon']

# Partition traits data for internal nodes
Tint <- T[T['down']!='.']
dim(Tint) <- c(length(Tint)/ncol(T),ncol(T))
colnames(Tint) <- colnames(T)
rownames(Tint) <- c(-1:-(nrow(Tint)))

# Recombine traits data, tips first then internals
T2 <- rbind(Ttips,Tint)
colnames(T2) <- colnames(T)

#parse vars and ses
ages <- as.numeric(Tint[,'age'])
X <- as.numeric(T2[,7])
Xse <- as.numeric(T2[,8])
Y <- as.numeric(T2[,9])
Yse <- as.numeric(T2[,10])

#calculation of mean ages based on ancml contrasts 
# ACx is vector of absolute contrasts
# ACageX = Wx in text of paper
ACx <- abs(picfixed(X,tree,scaled=FALSE))
ACageX <- weighted.mean(ages,ACx)

ACy <- abs(picfixed(Y,tree,scaled=FALSE))
ACageY <- weighted.mean(ages,ACy)

# Save observed contrasts, before running bootstrap reps
ACxObs <- ACx
ACyObs <- ACy

#calculation of correlations between contrasts and ages - not used
RageX <- cor(ACx,ages)
RageY <- cor(ACy,ages)

#calculation of mean ages based on Felsenstein contrasts
ifelse (BLzero == 0, 
	{
	# Contrasts based on Felsenstein's algorithm are used
	# to check assumptions of Brownian motion, and for significance
	# testing by alternative null models discussed in online archives
	Cx <- abs(pic(X[1:NTips],tree,scaled=FALSE))
	Cy <- abs(pic(Y[1:NTips],tree,scaled=FALSE))
	CageX <- weighted.mean(ages,Cx)
 	CageY <- weighted.mean(ages,Cy)
	CxObs <- Cx
	CyObs <- Cy
	Fx <- pic(X[1:NTips],tree,var.contrasts=TRUE)
	Fy <- pic(Y[1:NTips],tree,var.contrasts=TRUE)
	# Check if standardized contrasts are correlated with their SD
	BMx <- cor(sqrt(Fx[,2]),abs(Fx[,1]))
	BMy <- cor(sqrt(Fy[,2]),abs(Fy[,1]))
	# Check R value of truncated normal probability plot
	# low values indicate deviations from straight line
	PPx <- cor(qtnorm(qq,lower=0),sort(abs(Fx[,1])))
	PPy <- cor(qtnorm(qq,lower=0),sort(abs(Fy[,1])))
	# Store results
	result <- c(ACageX,ACageY,CageX,CageY,CageX,CageY,BMx,BMy,PPx,PPy)
	},
	# If zero branch lengths present, only store results from ACx
	result <- c(ACageX,ACageY))

# Store cumulative sums and sums of squares to calculate average 
# contrast values, and sd, over bootstrap samples
if (BATCH == 0) 
{
	cumACx <- ACx
	cumACy <- ACy
	cumACx2 <- ACx*ACx
	cumACy2 <- ACy*ACy
}
 
#create variables for bootstrap samples
Xr<-X
Yr<-Y

#run bootstrap samples and calculate agediff

for (r in c(1:Nreps)) 
{
	# bootstrap of ancml contrasts
	for (n in c((NTips+1):NTot)) 
	{
		#print(n)
		Xr[n] <- rnorm(1,X[n],Xse[n])
		Yr[n] <- rnorm(1,Y[n],Yse[n])
	}
	
	ACx <- abs(picfixed(Xr,tree,scaled=FALSE))
	RageX <- cor(ACx,ages)
	mnageX <- weighted.mean(ages,ACx)

	ACy <- abs(picfixed(Yr,tree,scaled=FALSE))
	RageY <- cor(ACy,ages)
	mnageY <- weighted.mean(ages,ACy)
	
	ifelse (BLzero == 0, 
	{
	# tip randomization and contrast randomization here
	Cx <- abs(pic(sample(X[1:NTips],replace=REPLACE),tree,scaled=FALSE))
	Cy <- abs(pic(sample(Y[1:NTips],replace=REPLACE),tree,scaled=FALSE))
	CageX1 <- sum(Cx*ages)/sum(Cx)
 	CageY1 <- sum(Cy*ages)/sum(Cy)
	# randomize contrasts
	Cx <- sample(CxObs,replace=REPLACE)
	Cy <- sample(CyObs,replace=REPLACE)
	CageX2 <- sum(Cx*ages)/sum(Cx)
 	CageY2 <- sum(Cy*ages)/sum(Cy)
	rand <- c(mnageX,mnageY,CageX1,CageY1,CageX2,CageY2,BMx,BMy,PPx,PPy)
	}, #end BLzero==0
	rand <- c(mnageX,mnageY))

	result <- rbind(result,rand)
	
	if (BATCH == 0)
	{
		cumACx <- cumACx + ACx
		cumACy <- cumACy + ACy
		cumACx2 <- cumACx2 + ACx*ACx
		cumACy2 <- cumACy2 + ACy*ACy
	}
}

# Add one for observed data
Nreps<-Nreps+1

#mean and standard deviation of ancml contrasts at each node
#ACx and ACy are now averages across all reps
if (BATCH == 0)
{
	meanACx <- cumACx/Nreps
	SDx <- (cumACx2-(cumACx^2/Nreps))/(Nreps-1)
	SDx[SDx<0] <- 0
	SDx <- sqrt(SDx)

	meanACy <- cumACy/Nreps
	SDy <- (cumACy2-(cumACy^2/Nreps))/(Nreps-1)
	SDy[SDy<0] <- 0
	SDy <- sqrt(SDy)

	#standardize contrasts relative to largest
	#SDx <- SDx/max(meanACx)
	#meanACx <- ACx/max(meanACx) 

	#SDy <- SDy/max(meanACy)
	#meanACy <- ACy/max(meanACy)
}

#calculate differences in mean age under bootstrap
agediff <- as.array(result[,1]-result[,2])
dim(agediff) <- c(length(agediff),1)
colnames(agediff) <- 'bootstrap'

#calculate one-tailed probability that ancml agediff is greater than zero
pAnc <- length(agediff[agediff<=0])

#calculate differences in age under null models
if (BLzero == 0) {
	pRand <- NULL
	for (i in 2:3) { 
		agediff <- cbind(agediff,result[,2*i-1]-result[,2*i]) 
		pRand <- cbind(pRand,rank(agediff[,i])[1])  
	}
	colnames(pRand) <- c('pTips','pContrasts')
	colnames(agediff)[2:3] <- c('tip.null','contrast.null')
}

# Not currently used
#calculate difference in correlations from each rep, and sig
Rdiff <- result[,3]-result[,4]
names(Rdiff) <- NULL
Rplow<-length(Rdiff[Rdiff<=0])

ifelse(BATCH == 1,
	{
	out <-  cbind(Nreps,result[1,1],result[1,2],
		agediff[1,1],mean(result[,1]),sd(result[,1]),mean(result[,2]),
		sd(result[,2]),mean(agediff[,1]),sd(agediff[,1]),pAnc,
		T[1,'beta1'],T[1,'beta2'])
	colnames(out) <- c("Nreps","Wx.as","Wy.as","D.as","Wx.bs",
			 "Wx.sd","Wy.bs","Wy.sd","D.bs","D.sd","P.bs",
			 "betaX","betaY") 
	if (BLzero == 0) 
	{
		out <-  cbind(out,result[1,3],result[1,4],agediff[1,2],
			pRand,result[1,7],result[1,8],result[1,9],
			result[1,10])
		colnames(out) <- c("Nreps","Wx.as","Wy.as","D.as","Wx.bs",
			 	 "Wx.sd","Wy.bs","Wy.sd","D.bs","D.sd","P.bs",
			 	 "betaX","betaY","Wx.pic","Wy.pic",
				 "D.pic","P.tips","P.contrasts",
				 "BMtestX","BMtestY","NormTestX","NormTestY")
	}
	outfile <- file("dot.out",open = "w")
	write.table(out,file=outfile,
		quote=FALSE,
		sep=',',
		row.names=FALSE,
		col.names=TRUE)	
	close(outfile)
	# for some indiscernable reason, R requires some command at this 
	# point or it generates an error - the following is a placeholder
	if (TRUE) BATCH <- BATCH
	},
	# if BATCH == 0
	{
		print("Bootstrap method results:")
		print("Wx (mean, sd)") 
		print(c(mean(result[,1]),sd(result[,1])))
		print("Wy (mean, sd)") 
		print(c(mean(result[,2]),sd(result[,2])))
		print("D (mean, sd)") 
		print(c(mean(agediff[,1]),sd(agediff[,1])))
		print("P = ") 
		print(pAnc/Nreps)
		print("")
		print("Randomization method results:")
		print("Wx, Wy")
		print(c(result[1,3],result[1,4]))
		print("D") 
		print(agediff[1,2])
		pR <- (Nreps-pRand+1)/Nreps
		colnames(pR) <- NULL
		rownames(pR) <- NULL
		print("P(tips) = ") 
		print(pR[1])
		print("P(contrasts) = ") 
		print(pR[2])
	})
